#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Author: Dean Harris (marcus_furius@hotmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import sys
import time
import os
import re
import config
import webbrowser
import constants
from gettext import gettext as _
from newsreader import NewsReader
try:
    import pygtk
    pygtk.require("2.0")
except:
    pass
try:
    import pango
    import gtk
    import gtk.glade
except:
    sys.exit(1)

class FuriusMicroRss:

    def __init__(self):
        #Set the Glade file
        self.wTree = gtk.glade.XML(constants.application_interface, 'MainWindow')
        #Next, create our treeview
        self.treeview = self.wTree.get_widget('treeview_news')
        self.setup_treeview(self.treeview)
        self.setup_widgets()
        self.setup_handlers()
        
        self.current_feed = 0
        self.read_news(config.feeds[self.current_feed])

    def setup_treeview(self, tree):
        """Sets up the treeview"""
        self.storage = gtk.ListStore(str)
        renderer = gtk.CellRendererText()
        self.treeview.set_model(self.storage)
        self.treeview.insert_column_with_attributes(-1, 'Headlines', renderer, markup=0)
        self.treeview.get_column(0).set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        #self.treeview.get_column(0).set_sort_column_id(0)
        self.treeview.modify_font(pango.FontDescription("sans 8"))
        
        
    def setup_widgets(self):
        """Sets up the glade widgets"""
        self.wTree.get_widget("MainWindow").set_icon_from_file(constants.file_image_icon)
        self.textview_overview_buffer = gtk.TextBuffer()
        self.wTree.get_widget("label_channel").modify_font(pango.FontDescription("sans 8"))
        self.wTree.get_widget("textview_overview").modify_font(pango.FontDescription("sans 8"))       
        self.wTree.get_widget("label_news_time").modify_font(pango.FontDescription("sans 8"))
        self.wTree.get_widget("label_last_update").modify_font(pango.FontDescription("sans 8"))
        self.wTree.get_widget("textview_overview").set_buffer(self.textview_overview_buffer)
        
        self.wTree.get_widget("button_previous").set_sensitive(False)
        if len(config.feeds) == 1:
            self.wTree.get_widget("button_next").set_sensitive(False) 
    
    def setup_handlers(self):
        """Sets up the glade handlers"""
        HANDLERS_AND_METHODS = {
            "destroy": self.destroy, 
            "on_treeview_news_cursor_changed":self.on_treeview_news_cursor_changed,
            "on_treeview_news_row_activated":self.on_treeview_news_row_activated,
            "on_button_next_clicked":self.on_button_next_clicked,
            "on_button_previous_clicked":self.on_button_previous_clicked, 
            "on_button_refreshed_clicked":self.on_button_refreshed_clicked,
            "on_button_about_clicked":self.on_button_about_clicked,
            "on_button_home_page_clicked":self.on_button_home_page_clicked}
            
        self.wTree.signal_autoconnect(HANDLERS_AND_METHODS)
        
    def read_news(self, url):
        self.storage.clear()
        self.textview_overview_buffer.set_text('')
        self.wTree.get_widget("label_news_time").set_text('')
        self.osn = NewsReader(url, True, -1)
        max_length = 32
        if len(self.osn.title) > max_length:
            self.osn.title = self.osn.title[:max_length] + "..."
        self.wTree.get_widget("label_channel").set_text(self.osn.title)
        for i in xrange(self.osn.itemcount):
            self.storage.append([self.osn.titles[i]])
                
        self.wTree.get_widget("label_last_update").set_text(time.asctime())
        
    def on_treeview_news_cursor_changed(self, data):
        #First get the tree view selection
        self.treeselection = self.treeview.get_selection()
        #Get the TreeModel and TreeIter of our selection
        (self.model, self.iter) = self.treeselection.get_selected()
        #Get the path of our selection as indices
        self.indices = self.model.get_path(self.iter)
        #Strip any Html tags
        try:
            text = re.sub(r'<[^>]*?>', '', self.osn.descriptions[self.indices[0]])
        except:
            text = 'Read full story for latest details.'
        self.textview_overview_buffer.set_text(text)
        self.wTree.get_widget("label_news_time").set_text(self.osn.times[self.indices[0]])

    def on_treeview_news_row_activated(self, iter, path, user_data):
        webbrowser.open(self.osn.links[path[0]])
        
    def on_button_next_clicked(self, data):
        self.current_feed += 1
        self.read_news(config.feeds[self.current_feed])
        self.wTree.get_widget("button_previous").set_sensitive(True)
        if len(config.feeds) == self.current_feed + 1:
            self.wTree.get_widget("button_next").set_sensitive(False)
        
    def on_button_previous_clicked(self, data):
        self.current_feed -= 1
        self.read_news(config.feeds[self.current_feed])
        self.wTree.get_widget("button_next").set_sensitive(True)
        if self.current_feed == 0:
            self.wTree.get_widget("button_previous").set_sensitive(False)
    
    def on_button_refreshed_clicked(self, data):
        self.storage.clear()
        self.textview_overview_buffer.set_text('')
        self.read_news(config.feeds[self.current_feed])
                                
    def on_button_about_clicked(self, data):
        """
        Show an about dialog box
        """
        about_dialog = gtk.AboutDialog()
        about_dialog.set_transient_for(self.wTree.get_widget("MainWindow"))

        # Set credit information
        about_dialog.set_name(constants.application_name)
        about_dialog.set_version(constants.application_version)
        about_dialog.set_website(constants.application_URL)
        about_dialog.set_translator_credits(_('translator-credits'))
        about_dialog.set_comments(_('A light weight, cross platform and portable RSS feed reader.'))
        
        about_dialog.set_authors([
            _('Developer:'),
        'Dean Harris <marcus_furius@hotmail.com>',
        '',
        _('Thanks to:'),
        ' * Dirceu Veiga (RSS Icons)',])

        # Set logo
        if os.path.exists(constants.about_image_icon) :
            about_dialog.set_logo(gtk.gdk.pixbuf_new_from_file(constants.about_image_icon))

        # Load the licence from the disk if possible
        if os.path.exists(constants.application_license) :
            about_dialog.set_license(open(constants.application_license).read())
            about_dialog.set_wrap_license(True)

        about_dialog.run()
        about_dialog.destroy()
          
    def on_button_home_page_clicked(self, data):
        webbrowser.open(self.osn.link)
        
    def destroy(self, data):
        gtk.main_quit()
        
if __name__ == "__main__":
    app = FuriusMicroRss()
    gtk.main()
